severity        = 3
verbose         = 6
top             = 50
theme           = pbp || core || bugs || security || maintenance
criticism-fatal = 1
color           = 1
allow-unsafe    = 1

[TestingAndDebugging::ProhibitNoStrict]
allow = refs
[Variables::RequireLocalizedPunctuationVars]
allow = @ARGV
[-Miscellanea::ProhibitUnrestrictedNoCritic]
[Variables::ProhibitPackageVars]
add_packages = Text::WrapI18N
[-Subroutines::ProhibitBuiltinHomonyms]
[Subroutines::ProhibitUnusedPrivateSubroutines]
allow=_options_sub_commands
